****************************************************************************************************************************************************************
*********     Introduction to Programming     **********
****************************************************************************************************************************************************************
/*
* Macros 

Reference:  Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire, Ch. 14;
     		Kohler, U. and Kreuter, F. (2005). Data Analysis Using Stata. Stata Press, Ch. 3, 11

/* Macros are names (up to 31 characters) that can stand for strings, 
program-defined numerical results or user-defined values.

A local macro exists only within the program (do-file) that defines it, 
and cannot be referred to in another program.

Once defined, global macros remain in memory and can be used by other programs 
for the duration of your current Stata session.

*  Local macros

// local macros are created by command local
// to acces created local macro you enclose the macro name in single quotes

* example

local ylist price weight length
summarize `ylist' 

Note that the left and right quote are different character! */

* example

local m1 2+2
display "`m1'"
display `m1'

local m2 = 2+2
display "`m2'"
display `m2'

* Display with quotation marks displays the test
* Hence, the first macro contains the expression, whereas the second contains the result

* Display without quotation marks shows the value of the expression

* Note a crucial difference when multiplying

local m1 2+2
display 2*`m1'

local m2 = 2+2
display 2*`m2'

* To create a local macro named "i", standing for the number 5, type

local i 5

* Thus, to increase the value of i by one, we write 

local i = `i' + 1
display `i'

local i 1
local i = `i' + 10
display `i'

local i 2
local i = `i' + `i'
display `i'

local i i is `i'
display "`i'"

/* Two commonly used macro-extension operators add or substract the numbber 1 from a macro,
either immediately before the Stata expands the macro or immediately therafter. */

local i 1
display `i++'
display `i'

/* Here we first define the local macro i.
Afterwards we display i with an extension operator (++),
meaning that the macro is expanded, and then one is added to the macro;
that is the content of the macro is changed to 2.
If you put the extension operator in front of the macro name, 1 is added to the macro 
before the macro is expanded.
The same applies to the extension operator -- for substracting 1: */

local i 1
display `i++'
display `i'
display `++i'
display `i--'
display `--i'

* Instead of a number, the macros contents could be a string or list of words, such as

local islands Iceland Faroes

* To see the string contents, place double quotes around the single-quoted macro name:

display "`islands'"

* We can concatenate further words or numbers, adding to the macros contents. For example,

local islands `islands' Newfoundland Nantucket
display "`islands'"

* Type help extended fcn for information about Statas extended macro functions, 
// which extract information from the contents of macros. 
// For instance, we could obtain a count of words in the macro, a
// and store this count as a new macro named howmany:

local islands Iceland Faroes
local islands `islands' Newfoundland Nantucket
local howmany: word count `islands'
display `howmany'

* Many other extended macro functions exist, with applications to programming.

* # Global macros

/* global macros are created by command "global"; 
to access created macro we need to put $ before the macro name

* example

global xlist price weight length
summarize $xlist

*/

global distance = 73
display $distance * 2

/* Unless we specifically want to keep macro contents for re-use later in our session, it is better
(less confusing, faster to execute, and potentially less hazardous) if we use local rather than
global macros in writing programs. */

* To drop a macro from memory, issue a "macro drop" command.
macro drop distance

* We could also drop all macros from memory.
macro drop _all

* Note that "claer all" does not delete macros
****************************************************************************************************************************************************************

************ Looping ***************
****************************************************************************************************************************************************************

/* 
Reference: Kohler, U. and Kreuter, F. (2005). Data Analysis Using Stata. Stata Press, Ch. 3

/* sometimes we can make our life easier by using loops in stata for situations 
when we need to run one command for many variables; 

in general, we can use the by prefix, foreach loop and forvalues loop; 

in many situations loops are substitutable up to a point by the by-prefix; 
meanwhile the forvalues loop is specific case of foreach loop applied to numerical lists (numlists)*/

* # by prefix

/* The by prefix repeats a command for every group of observations for which the values in the
variable list are the same.
Most Stata commands accept the by prefix;
this is indicated immediately following the syntax diagram and options table, for example,

by may be used with summarize

The by prefix consists of two words "by" or "bysort", a variable list (bylist), and a colon.
Using the by prefix generally requires that the dataset be sorted by the variables of the bylist,
but there are two ways to sort the data on the fly.

Summarize the income for each group of the variable gender.
First, we need to sort the data by gender and then issue summarize with the by prefix: */

use data1.dta, clear
sort gender
by gender: summarize income

by edu, sort: summarize income
bysort edu: summarize income

/* If you provide more than one variable in the bylist, Stata does what you have told it to do
for each possible combination formed by the values of the bylist variable.

Therefore, the command */

by gender edu, sort: summarize income

/* first has Stata summarize the income of all men with a minimum of 9 years of education ("elementary").
Then it does the same for all men with 10 years of education ("intermediate") and so on.
After summarizing the income of all educational groups of males,
it does the same for each educational group of females. */
****************************************************************************************************************************************************************

* # foreach loop

/* the foreach loop is used to repeat a specific command for each element of a list;
this task can be one or set of commands */

/* A simplified syntax of foreach:

foreach lname listtype list {
	commands
}

*/

/* foreach loop has always at least three lines:

	- first line starting with command "foreach", which start the loop, and is 
	followed by element name ("lname") (which is used throughout the loop to refer to element
	of the list), list type ("listtype") (which specifies what elements we have, e.g. varlist for
	list of variables) and the foreach list (list); the last sign on the first line is open
	brace (i.e. curly brackets)
	
	- in second line (or more lines) we include the commands of the statement, which
	use the element name to refer to one of the elements of the list
	
	- on last line we include close brace */

* ## The types of foreach lists

/* There are 4 types of list:

	- varlist (used with preposition of), specifying list of variables
	- newlist (used with preposition of), specifying list of new variables, which
	will be created by Stata during the loop
	- numlist (used with preposition of), specifying list of numbers; note that 
	the list can include only one range
	- arbitrary list (used with preposition in), which can include letters, words 
	and numbers seperated by spaces */
	
* examples

use data1, clear 
describe

foreach X of varlist np9501-np9507 {
		tabulate `X' gender
		}
		
/* note that we used the element name "X" and when we reffered to it in the loop, 
we enclosed it in single quotation marks, which is the sign for local marco in 
Stata. When Stata encounters `X' in the commands it successivly fills in the 
elements from our list. We specified that our list is list of variables and we 
included 6 variables. */

* example with newlist

drop _all

set obs 1000
set seed 12345

foreach var of newlist r1-r10 { 		//element name is "var"
		generate `var' = runiform()		// succesive loops will look like "generate r1 = rnormal()"
										// "generate r2 = rnormal()" and so on
		}
		
summarize	

/* Next we use "of numlist" to replace the variables with newly generated random numbers.
As the variables r1 to r10 already exist, we need to use replace instead of generate within the loop: */	
		
* example with numlist

foreach x of numlist 1/10 {
		replace r`x' = rnormal() // succesive loops will look like "repalce r1 = rnormal()
								 // "replace r2 = rnormal() and so on	
		}
		
summarize	

/* crucially note that we can include the element name inside of other words like
variable names or even commands; simply, they work like similarly to macros */

* example with an arbitrary list

foreach piece in This list has 5 pieces {
		display "`piece'"	
		}

use data1, clear 
describe 
	
foreach piece in edu emp {
		table `piece' gender
		}

* ## Several commands within a foreach loop

/* You can put more than one command within a foreach loop.
In the next example, we generate a centered version of a variable for income and year of birth
To produce a centered version of a variable, we need to subtract its arithmetic mean 
from each of its values: */

foreach var of varlist ybirth income {
		summarize `var', meanonly
		generate `var'_c = `var' - r(mean)
		label variable `var'_c "`var' (centered)"
		}
		
summarize ybirth_c income_c

/* We begin by calculating the arithmetic mean, and then we generate a new variable
having the name of the old variable with _c appended.
The term r(mean) refers to a "saved result" of the summarize command, 
which contains the value of the mean we just calculated.
Finally, we define a variable label for each of the new variable. */
****************************************************************************************************************************************************************

* # forvalues loop

/* forvalues loop is simplified version of foreach loop which is applicaple only
to numerical lists; this means we can ommit the type of list. */

/* A simplified syntax of forvalues:

forvalues lname = range {
	commands
}

*/

/* In the first line of the syntax diagram, you find the command itself - forvalues - 
followed by an element name (lname), an equal-sign, a range, and then the opening brace.
The range is similar to a numlist. 
You cannot list single numbers or more than one range. 
For example, you can specify 1(1)10 for all integer numbers from 1 to 10, 
but you cannot specify 1(1)10 15 19 or 1(1)10 12(2)20 

The element name is an arbitrary name for each number of the range. */

* example

foreach var of newlist r1-r10 { 		//element name is "var"
		generate `var' = runiform()		// succesive loops will look like "generate r1 = rnormal()"
										// "generate r2 = rnormal()" and so on
		}

forvalues num = 1/10 {
		replace r`num' = runiform()
		} 

/* num is enclosed in single quotes just like the element name in the foreach loop.
After you type the third line, Stata should begin the loop, successively replacing
`num' with each number of the specified range: 

The forevalues loop may seem unnecessary, given that it is the same as a foreach loop
with the list type of numlist.
However, foreach with a numlist has some limitations;
it cannot have more than 1600 numbers in the list.
There is no such limitation with forvalues. Moreover, forvalues is more efficient for 
Stata process. */
****************************************************************************************************************************************************************
****************************************************************************************************************************************************************

********** Stata programs ***********

* # simple Stata programs

/* in Stata programs are written in similar fashion to loops. 
we start them by writting command "program" (or equvivalent "program define")
followed by the name of the program (no longer that 31 characters). 
On following lines we then write standard stata commands like in loops. 
We finish the program by command "end" */

* exmaple

clear all

* Write a program and run it

program simul_normal
	capture drop y
	set obs 100
	generate y = rnormal()
end
 
* To execute the program simply write the name of the program in as command
 
simul_normal

/* Note that running program displays only the result - 
an advantage of programs when compared to loops */

/* One should never define a program with the same name as a biult-in command.
To ascertain whether a command with a given name already exists, enter "which commandname"
If Stata replies with "Command commandname not found as either biult-in or ado-file",
one may use the name for her program. */

/* Once program is written in the command window and executed, Stata stores it in its internal memory 
from where it can be recalled untill deleted. 
Stata deletes it if we execute "clear all" or "program drop programnamr" command or if we close stata. 
This is the advantage and disadvantage of Stata programs - 
they are in their nature temporary. 

We can solve this by saving the programs as do or ado files */

/* Sometimes we make mistake in programs; this can be retraced by command
"set trace on", which instracts Stata to display programs commands */ 

* example

drop y

set trace on
simul_normal
set trace off
****************************************************************************************************************************************************************

**** Programs in do-files *****

/* To define the program in do-file, one can write a simul_normal.do file as follows:

program simul_normal
	capture drop y
	set obs 100
	generate y = rnormal()
end
exit

Now that once the program definition is written into the do-file we do not have to write the definition
of the program every time we close stata; executing the do-file is sufficient.

If the program has already been defined interactively, then it is currently stored in memory 
and cannot be overwritten.
Therefore, one must delete this older version before creating the new version.
One can type "program drop simul_normal".
However, this is best done directly in the do-file through "capture program drop".

Capture in front of a command makes Stata indifferent to whether the command works.
Without capture if we would run the do-file first time in a Stata session, it would fail.

capture program drop simul_normal
program simul_normal
	capture drop y
	set obs 100
	generate y = rnormal()
end
exit

Now you have to save and execute the do-file. The program will not be executed 
as it has been stored in memory only through the command in the do-file.
However, one can now execute the program interactively. */

drop y

do simul_normal
simul_normal

/* One can also call up the program directly in the do-file as follows:

capture program drop simul_normal
program simul_normal
	capture drop y
	set obs 100
	generate y = rnormal()
end
simul_normal
exit

One just need to type "do simul_normal" to run the program. */

drop y

do simul_normal

/* Saved do-file can also be executed with "run" command.
The command run is the same as the do command, the only difference being that
after you enter run, the individual command lines do not appear on the screen. */

drop y

run simul_normal

* ## Programs in ado-files

/* Consider the program written as follows:

program simul_normal
	set obs 100
	generate y = rnormal()
end
exit

If we save a do-file with the extension .ado instead of .do,
entering the name of the ado-file after we drop the program (and y in our case) 
from the memory would be sufficient to define and execute the program
(note, we do not need to call up the program directly in the ado-file to execute it
as we needed to do in the do-file). */

drop y
program drop simul_normal

* Create simul_normal.ado file

simul_normal

/* To understand why the programs that are defined by ado-files must be deleted from memory 
before any potential changes to the program take effect, consider the Stata command search steps
(pay attention to the second step):

1. Stata checks if the command is an internel command. If so, then Stata will have it executed.
If not, Stata moves on to the next step.
2. Stata checks if the command is stored in memory. If so, the program would be executed.
If not, Stata moves on to the next step.
3. Stata searches for the commandname.ado.

To delete the old program from the memory before the ado-file can be reimported,
one can use "program drop" or "discard" command.
"discard" forces Stata to drop all automatically loaded programs from memory and reload them from disk
the next time they are called. */

drop y
discard
simul_normal

/* We do not need to type "capture program drop simul_normal" at the beginning of ado-file,
because if we are not sure whether the program in currently in Stata memory and try to run it,
then, if the program is in memory, it will be run;
if the program is not in the memory, Stata will refer to the commandname.ado file and execute
the program.
We will not get any error message anyway. 
If we do not have ado-file with defined program, we will do (run) respective do-file. */

* ## advanced Stata programs 

/* Stata allows you to write more comlpicated programs than simple collection of commands; 
most relevant for us is using temporary variables, named positional arguments 
and programs with Stata syntax */

/* If you write a substantial program, it will invariably require the use of temporary variables in the data, 
or temporary scalars, matrices, or files. 
Temporary objects are necessary while the program is making its calculations, 
and once the program completes they are discarded. */

/* Stata provides three commands to create temporary objects: 
tempvar creates names for variables in the dataset, 
tempname creates names for scalars and matrices, 
and tempfile creates names for files. */

/* We recall temporary objects like local macro with single quotes */

clear all

* example

capture program drop normal_simul_2
program simul_normal_2 
	set obs 100
	tempvar x // first we state that we want to create temporary variable x
	generate `x' = rnormal() // we refer to x as to local macro
	summarize `x'
	display r(mean)
end

simul_normal_2 // the program does not create x in variables

/* similarly we can create temporary scalars and matrices with command "tempname"
and files with command "tempfile" */

*** Program arguments [based also on User's Guide]

/* When you invoke a program or do-file, what you type following the program or do-file name 
are the arguments. For instance, if you have a program called xyz and type

xyz mpg weight

then mpg and weight are the programs arguments, mpg being the first argument 
and weight the second. Program arguments are passed to programs via local macros. */

/* The first option is to use the positional arguments, either named or unnamed; 

in case of unnamed positional argumetns we refer to them in the program 
by local macros `1' `2' and so on. */

* examples

clear all

program xyz
	correlate `1' `2'
end

use auto.dta, clear
xyz mpg weight

/* In case of named positional arguments we first define them by comand "args" 
followed by the names of the arguments, which become their local macros. */

clear all

capture program drop normal_simul_3
program normal_simul_3
	args n m std
	set obs `n'
	generate z = rnormal(`m',`std') // arguments m and std passed as mean and 
	summarize z						// standard deviation equal to std
end									
	
normal_simul_3 100 2 4 // generates normal variable with mean 2 and standard deviation 4, and the sample size of 1000

* example

capture program drop normal_simul_4
program normal_simul_4
	args n name m std 
	set obs `n'
	generate `name' = rnormal(`m',`std') // in addition uses allows to choose name
	summarize `name'
end
	
normal_simul_4 100 w 2 4 // creates w with mean 2 and standard deviation 4

/* The most advanced option is specifying the full syntax like normal Stata commands have; 
this is done by command syntax followed by anything from standard Stata syntax  */

* example 

/* We will create program which will generate normally distributed variable as 
before, but will allow us OPTIONALLY to change the mean and standard deviation,
with defining the default values to be 0 and 1 */

clear all

capture program drop simul_normal_5
program simul_normal_5
	syntax [, mu(real 0) sigma(real 1)]
	/* creates options for mean - has to be real number, default 0 - 
	and standard deviation - has to be real number, default 1 */
	set obs 100
	tempvar y
	generate `y'=rnormal(`mu',`sigma')
	sum `y'
end

simul_normal_5

simul_normal_5, mu(10) sigma(10)

/* We can also make program store its results; for this we have to specify
that the program itself is r-class or e-class program and define the return scalars */
	
* example

/* this program returns the mean from generated random variable as in simul_normal_4 */

capture program drop normal_simul_6
program normal_simul_6, rclass
	args n name m std
	drop _all
	set obs `n'
	generate `name' = rnormal(`m',`std') 
	summarize `name'
	tempname x  // defines temporary scalar
	scalar `x' = r(mean) // assigns value to the temporary scalar
	return scalar est_mean = `x' // assigns value of x to the return scalar est_mean
end 	

normal_simul_6 100 xy 5 10
return list // displays return list for program normal_simul_6
****************************************************************************************************************************************************************

***** while command in looping *****

* [based on Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire, Ch. 14]

/* forvalues and foreach create loops that repeat for a pre-specified number of times. 
If we want looping to continue until some other condition is met, the while command is useful. 
A section of program with the following general form will repeatedly execute the commands 
within curly brackets, so long as expression evaluates to true:

while expression {
		command A
		command B
. . . .
}
command Z

When expression evaluates to false, the looping stops and Stata goes on to execute command Z. 
Here is a simple program that uses a while loop to display onscreen the iteration numbers from 1 through 6:

Program that counts from one to six */

clear all

capture program drop count6
program count6
	local iterate = 1
	while `iterate' <= 6 {
		display `iterate'
		local iterate = `iterate' + 1
}
end
count6

* ## If...else in looping

/* The if and else commands tell a program to do one thing if an expression is true, 
and something else otherwise. 
They are set up as follows:

if expression {
		command A
		command B
. . . .
}
else {
		command Z
}

For example, the following program segment checks whether the content of local macro span
is an odd number, and informs the user of the result.

if int(`span'/2) != (`span' - 1)/2 {
		display "span is NOT an odd number"
}
else {
		display "span IS an odd number"
}

*/


*** Examples based on Kohler, U. and Kreuter, F. (2005). Data Analysis Using Stata. Stata Press, Ch. 11
****************************************************************************************************************************************************************

****** User-written Stata commands *******

clear all

/* Let's see how one can generate a graph with superimposed kennel density estimates. 
*/
use data1.dta, clear
describe
gen linc = log(income)
gen xhelp = log(_n*250) in 1/50
kdensity linc if gender == 1, generate(xvals1 fmen) at(xhelp) nograph
kdensity linc if gender == 2, generate(xvals2 fwomen) at(xhelp) nograph
graph twoway connected fmen fwomen xhelp ///
, title(Income by Gender) ytitle(Density) xtitle(Log (Income)) ///
legend(label(1 "Men") label(2 "Women"))
exit
/*
* generate(newvar_x newvar_d) -- stores the results of the estimation. newvar_x will contain the points
at which the density is estimated. newvar_d will contain the density estimate.

* at(var_x) -- specifies a variable that contains the values at which the density should be estimated.
This option allows you to more easily obtain density estimates for different variables or different
subsamples of a variable and then overlay the estimated densities for comparison

* nograph suppresses the graph. This option is often used with the generate() option.

The following section will describe how to generate such graphs with one command
for arbitrary datasets.
The command shoudl have roughly the following syntax:

denscomp varname [if] [in], by(varname) at(varname) [twoway_options]

That is, to produce our example graph,
varname is substituted with linc,
by(varname) becomes by(gender),
and at(varname) becomes at(xhelp), respectively.

Before you begin to program the generalization of the do-file, you should create
the first version of the ado-file.
This first version is almost identical to the above denscomp.do.
You can save denscomp.do with the name denscomp.ado and make the following changes:

1. Delete the use command, as well as both generate commands used to generate
the variables linc and xhelp.
2. Type the command program denscomp in the first line of the file.
3. Type the command version 10 in the second line of the file. 
This guarantees that your program will work error free in later Stata versions.
4. Type the command end to indicate the end of the program in the next-to-last line
of the file. The command exit ends the ado-file, not the program.

The preliminary version of denscomp.ado should look like this:

program denscomp
version 10
kdensity linc if gender == 1, generate(xvals1 fmen) at(xhelp) nograph
kdensity linc if gender == 2, generate(xvals2 fwomen) at(xhelp) nograph
graph twoway connected fmen fwomen xhelp ///
, title(Income by Gender) ytitle(Density) xtitle(Log (Income)) ///
legend(label(1 "Men") label(2 "Women"))
end
exit

Check whether everything still works after every major step.

If you run denscomp, you will now receive an error message: */

denscomp
* xvals1 already defined

/* You have to first delete the variables xvals1 & fmen and xvals2 & fwomen 
before you run denscomp.ado */

drop xvals1 xvals2 fmen fwomen
denscomp

/* We have created the graph with one command; however, denscomp will only create the graph 
with the same variable and options. Now we will generalize denscomp.

* ## Parsing variable lists

The first step in generalizing denscomp.ado 
is to allow the graph to be displayed for arbitrary variables.

This can be achieved with the syntax command.
The syntax command is used to define the structure with which a user is to call our program. 
Here we assume that the program is accessed through the name of a command and a variable name. 
This setup is parsed to the program in the following program way:

program denscomp
version 10
syntax varname(numeric)
kdensity `varlist' if gender == 1, generate(xvals1 fmen) at(xhelp) nograph
kdensity `varlist' if gender == 2, generate(xvals2 fwomen) at(xhelp) nograph
graph twoway connected fmen fwomen xhelp ///
, title(Income by Gender) ytitle(Density) xtitle(Log (Income)) ///
legend(label(1 "Men") label(2 "Women"))
end
exit

The syntax command has two functions: verifying input and parsing program arguments.

That is, syntax checks whether the user's input makes sense 
and either produces an error message or parses the input so the program can use it.

Let us clarify. As we code "syntax varname", Stata tries to interpret everything 
written behind denscomp as a variable name. 
If denscomp is called up without a variable name, State will issue an error message, 
indicating that a variable name is required.
Moreover, State will check whether the user gave one or more variable names. 
As we specified syntax varname, State will produce an error message if the user accidentally
specified more than one variable name. 

If we had decided to allow the used to specify more than one variable name, 
we would have coded "syntax varlist", with the effect that the user could have specified 
a list of variable names. 

Finally, as we specified "syntax varnane(numeric)", State will check whether the variable input 
by the user is a numeric variable or a string variable. 
If it is a string variable, State will issue an error message and exit the program immediately.

The second function of syntax is to parse program arguments given by the user.
The syntax command in our example causes State to save the entered variables name 
in the local macro varlist when the program is called up. 
With this in mind, we can use `varlist' in our program instead of the hard-coded variable name linc.

We now call this program using denscomp linc. 
You will first have to save denscomp.ado and then delete the old version of the program from
memory with discard and delete the variables xvals1 xvals2 fmen fwomen.  
Afterward, your program should run free of errors. */

discard
drop xvals1 xvals2 fmen fwomen
denscomp linc

* ## Parsing options

/* These changes have made the program more generalized. 
However, the distributions of men and women are still being compared with each other. 
We should be able to have the option of using the graph to compare other subgroups. 
We would like bo have an option we could use to define the variable whose values are to be compared. 

For example, this option could be called "by(varname)". 
The question that remains is how to transfer these options to the program.

For this, we again turn to the syntax command. 
As indicated above, by using syntax we are informing a program about the structure of the command. 

To enable the option by(uarname), we amend the syntax command as follows:

program denscomp
version 10
syntax varname(numeric), by(varname)
kdensity `varlist' if `by' == 1, generate(xvals1 fmen) at(xhelp) nograph
kdensity `varlist' if `by' == 2, generate(xvals2 fwomen) at(xhelp) nograph
graph twoway connected fmen fwomen xhelp ///
, title(Income by Gender) ytitle(Density) xtitle(Log (Income)) ///
legend(label(1 "Men") label(2 "Women"))
end
exit

With these changes, the program denscomp now expects to be called with an option
after the name of a numeric variable. 
We add the option by placing a comma in the syntax command. 
As we did not place the list of options in square brackets, we must specify the option
(we refer to a list of options even though currently only one option is allowed).  

If we had used "syntax varname(numeric) [, by(varname)]" the option would not have been required. 
Square brackets in syntax commands work just as described in the syntax diagram in the online help.

As it stands, we only allow one option: by(). 
Only one variable name is allowed inside the brackets of by(). 
There is no restriction on numeric variables; hence in this case, string variables are also allowed. 

syntax will place the variable name given in by() in the local macro called by. 

You can apply the same logic for the at() option in the two kdensity commands. 
The kdensity commands are both called with at(xhelp). 
However, this will work only for datasets that contain the variable xhelp. 
What we need instead is an option for denscomp to declcare the name of the numerical at() variable. 

Therefore, we type:

program denscomp
version 10
syntax varname(numeric), by(varname) at(varname numeric)
kdensity `varlist' if `by' == 1, generate(xvals1 fmen) at(`at') nograph
kdensity `varlist' if `by' == 2, generate(xvals2 fwomen) at(`at') nograph
graph twoway connected fmen fwomen `at' ///
, title(Income by Gender) ytitle(Density) xtitle(Log (Income)) ///
legend(label(1 "Men") label(2 "Women"))
end
exit

We used the keyword numeric in the at() option because kdensity can compute densities 
only at numerical points. 
State will issue an error message if the user specifies a string variable. 

To try out these changes, save your file, and type the following commands in Stata. 
You need to use the discard command to remove the current ado-file from memory. 
Only then the changes become effective. */

discard
drop xvals1 xvals2 fmen fwomen
denscomp linc, by(gender) at(xhelp)

/* You should now see the familiar graph. If not, carefully check your file for any errors.
To determine which of the program lines is faulty, you can use the command "set trace on".

* ## Parsing if and in qualifiers

Let's see how to include if and in qualifiers in the program. 
Once again, our starting point is syntax:

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric)
marksample touse
kdensity `varlist' if `by' == 1 & `touse', generate(xvals1 fmen) at(`at') nograph
kdensity `varlist' if `by' == 2 & `touse', generate(xvals2 fwomen) at(`at') nograph
graph twoway connected fmen fwomen `at' ///
, title(Income by Gender) ytitle(Density) xtitle(Log (Income)) ///
legend(label(1 "Men") label(2 "Women"))
end
exit

Entering [if] and [in] lets you use if and in qualifiers. 
As both of them are in square brackets, neither element is required. 

Again syntax saves the qualifiers that have been entered by the user in local macros - 
here, in if and in, respectively. 
In doing so, the whole expression, including the keywords if and in, is saved in the macro.

So far, we have enabled if and in qualifiers. 
That is, State will not print an error message when the user invokes donscomp with an if qualifier. 

Now we must make our program exclude observations that do not satisy those conditions. 
One solution is to use the keep command to delete all the observations for which 
the preconditions are not applicable and only execute the commands for the remaining observations.

A better solution would be to use "marksample". 
marksample generates a temporary variable, and in our example the name of this temporary variable 
is "touse" (short for: to use)
In this variable, all cases that are to be used are assigned the value 1, 
and all other cases are assigned the value 0. 

marksample receives the information about which cases to use from the if and in qualifiers.

If the temporary variable touse has been defined by marksample, 
this variable can be used to restrict the statistical commands in the program to the observations 
marked with 1. 

Because touse is a temporary variable, it is placed between single quotation marks 
just like a local macro. In fact, louse is a local macro containing the name of the temporary variable. 

And you do not need to specify if 'touse'==1, as touse contains only 0 and 1, 
which is State's way to express false and true.


* ## Generating an unknown number of variables

Until now, we have only specified the basic characteristics of the ado-file. 
Now we will generalize our command. 

First, start the ado with: */

discard
drop xvals1 xvals2 fmen fwomen
denscomp linc, by(emp) at(xhelp)

/* This time, we have used employment status as the variable in the option by().
Something is not quite right here. 
The graph shows only two density curves, even though we were expecting one for every type
of employment status, which would mean a total of six curves. 
According to the labeling, the graph still shows the income of men and women, 
instead of the income of individuals with varying employment status.

Apparently, there is still some work to be done. 
We should start with the first problem: why are only two density curves being displayed, instead of six? 
The cause lies in kdensity lines of the program;

These lines contain the calculations for the density of the values 1 and 2 for the
variable given in by(). 
If we type the command with the option by(emp), the local macro by is replaced by emp 
and the densities for the first two values of employment status are displayed. 
There are no commands for any further categories.

The problem, however, is that the number of categories can vary with the variable
specified in by(). 
This in turn means that we do not know how often we want the program 
to execute the kdensity command.

If we knew the number of categories, we could create a loop in our program that would 
execute kdensity for every variable in by() and stop when there are no remaining categories. 

Therefore, we must somehow find out how many categories the variable in by() has.
The easiest way to get this information is to use the command "levelsof".

In general, levelsof is used to look into a specific variable and to store the existing
distinct values of this variable in a local macro. 
The variable emp, for example, contains values of 1-7, although the value 5 is not used. 

The following command will store these values into the local macro K: */

levelsof emp, local(K) 
display "`K'" 

/* Now let us see how you can use this in our program. 
We put quietly in front of the command.
quietly can be used in front of any Stata command to suppress all its output.

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric)
marksample touse
quietly levelsof `by' if `touse', local(K)
foreach k of local K {
	kdensity `varlist' if `by' == `k' & `touse', gen(xvals`k' fx`k') at(`at') nograph
}
graph twoway connected fmen fwomen `at' ///
, title(Income by Gender) ytitle(Density) xtitle(Log (Income)) ///
legend(label(1 "Men") label(2 "Women"))
end
exit

We used levolsof to store the distinct values of the by() variable in the local macro K. 
Then we set up a foreach loop over all elements of the local macro K. 
Within the loop, kdensity is executed for each employment category. 

We changed the option gen() of the kdensity command. 
Instead of generating hard-coded variable names xvalsl, xvals2, fmen, and fwomen, 
we generate variables with names according to the element name of the loop. 

That is, if we specify denscomp with the option by(emp), we generate variables 
xvalsl, xvals2, xvals3, xvals4, xvals5, xvals6, and xvals71 
and fxl, fx2, fx3, fx4, fx6, and fx7 since there are no observations for which emp is equal to 5.

Next, because the number of fx variables is no longer fixed, 
we have to adapt the graph command accordingly. 

For this reason, we create a list with the names of the generated fx variables 
within the loop and then save it in a local macro called yvars:

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric)
marksample touse
quietly levelsof `by' if `touse', local(K)
foreach k of local K {
	kdensity `varlist' if `by'==`k' & `touse', gen(xvals`k' fx`k') at(`at') nograph
	local yvars "`yvars' fx`k'"
}
graph twoway connected fmen fwomen `at' ///
, title(Income by Gender) ytitle(Density) xtitle(Log (Income)) ///
legend(label(1 "Men") label(2 "Women"))
end
exit

We can now include the local macro yvars in the graph command instead of fmen and fwomen:

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric)
marksample touse
quietly levelsof `by' if `touse', local(K)
foreach k of local K {
	kdensity `varlist' if `by'==`k' & `touse', gen(xvals`k' fx`k') at(`at') nograph
	local yvars "`yvars' fx`k'"
}
graph twoway connected `yvars' `at' ///
, title(Income by Gender) ytitle(Density) xtitle(Log (Income)) ///
legend(label(1 "Men") label(2 "Women"))
end
exit

Now it's time for another search for any errors: */

discard
drop xvals1 xvals2 fmen fwomen
denscomp linc, by(emp) at(xhelp)

/* It now looks a lot better. Nevertheless, the labeling of the graph remains a problem. 

* ## Default values

The first question is how to label the graph. 
We suggest that you make the default title "variable by by variable". 
"Density" would be suitable as the title of the y axis, 
and the title of the x axis should contain the variable name. 
The meaning of the data area would be explained in the legend by the categories of the by variable.

Let us start with the title of the graph. A simple way of achieving the desired result
is to change the graph command:

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric)
marksample touse
quietly levelsof `by' if `touse', local(K)
foreach k of local K {
	kdensity `varlist' if `by'==`k' & `touse', gen(xvals`k' fx`k') at(`at') nograph
	local yvars "`yvars' fx`k'"
}
graph twoway connected `yvars' `at' ///
, title("`varlist' by `by'") ytitle("Density") xtitle("Log (Income)") ///
legend(label(1 "Men") label(2 "Women"))
end
exit

This, however, has the disadvantage of being tied to our example. 
To be able to change the title when calling up the program, we first have to include 
the corresponding option in the syntax statement at the beginning of the program:

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric) [TItle(string asis)]
marksample touse
quietly levelsof `by' if `touse', local(K)
foreach k of local K {
	kdensity `varlist' if `by'==`k' & `touse', gen(xvals`k' fx`k') at(`at') nograph
	local yvars "`yvars' fx`k'"
}
graph twoway connected `yvars' `at' ///
, title("`varlist' by `by'") ytitle("Density") xtitle("Log (Income)") ///
legend(label(1 "Men") label(2 "Women"))
end
exit

We allow the user to specify an option called title(), 
where an arbitrary string can be specified, which is pulsed inside the program asis;
this means that quotation marks are part of the macro contents. 
The user now can enter a title when calling up the program. 
As we do not want to be obliged to enter a title, we have written the option in square brackets. 
The two capital letters indicate the minimum abbreviation, 
so the user can specify ti() instead of title().

As was the case before, the syntax command saves everything within the parentheses
in the local macro title. 
We can use this local macro in the title option of the graph command:

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric) [TItle(string asis)]
marksample touse
quietly levelsof `by' if `touse', local(K)
foreach k of local K {
	kdensity `varlist' if `by'==`k' & `touse', gen(xvals`k' fx`k') at(`at') nograph
	local yvars "`yvars' fx`k'"
}
graph twoway connected `yvars' `at' ///
, title(`title') ytitle("Density") xtitle("Log (Income)") ///
legend(label(1 "Men") label(2 "Women"))
end
exit

Now we no longer have a default title, so we will add one. 
Add the following line to the program just before the graph command:

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric) [TItle(string asis)]
marksample touse
quietly levelsof `by' if `touse', local(K)
foreach k of local K {
	kdensity `varlist' if `by'==`k' & `touse', gen(xvals`k' fx`k') at(`at') nograph
	local yvars "`yvars' fx`k'"
}
local title = cond(`"`title'"' == `""', `"`varlist' by `by'"', `"`title'"')
graph twoway connected `yvars' `at' ///
, title(`title') ytitle("Density") xtitle("Log (Income)") ///
legend(label(1 "Men") label(2 "Women"))
end
exit

Two things are new in this line: the function cond() and the compound quotes.
Let us first explain the cond() function. 
The function requires three arguments, each separated by a comma (cond(a,a,b)). 
The first argument contains a condition that will be evaluated. 
If this condition is true, the expression in the second argument is returned, 
whereas if the condition is false, the expression in the third argument is returned. 

Consider the following simplified version of the cond() function:

cond("`title'" == "", "`varlist' by `by'", "`title'"). 

Now think of a user who specified no title option. 
Then the first argument of the condition would become "" == "". 
This condition is true, so the second argument is returned: local title = "`varlist' by `by'". 
Hence, we will have a default title.
If the user specifies title(My title), the first argument becomes "My title" == "". 
This condition is false. Here Stata returns the third argument (local title = "My title"), 
which sets the title to the user-specified string.

Now for the compound quotes. Consider a user who includes title ( "My" "title") when invoking denscomp. 
That is, this user wants the word "My" as the first line of the title, and "title" as the second. 
Again the first argument in the cond() function will be false. 
However, this time, the simplified version of the line will become local title = ""My" "title"". 
The problem here is that Stata cannot distinguish the opening quotation
in front of "My" from a closing quotation mark. 

In fact, Stata cannot distinguish opening and closing quotation marks at all. 
To differentiate between the beginning of a quotation and the end of a quotation, we use compound quotes. 
To start a quotation, we use `" (a single opening quotation, followed by a double quote) 
and to end a quotation we use "' (a double quote, followed by a single closing quotation). 
We recommend that you always use compound quotes inside programs. 
They never cause problems but are often necessary.

The same procedure may be used for the other characteristics of the graph, 
e.g., the title of the y and x axes:

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric) ///
[TItle(string asis) YTItle(string asis) XTItle(string asis) * ]
marksample touse
quietly levelsof `by' if `touse', local(K)
foreach k of local K {
	kdensity `varlist' if `by'==`k' & `touse', gen(xvals`k' fx`k') at(`at') nograph
	local yvars "`yvars' fx`k'"
}
local title = cond(`"`title'"' == `""', `"`varlist' by `by'"', `"`title'"')
local ytitle = cond(`"`ytitle'"' == `""', `"Density"', `"`ytitle'"')
local xtitle = cond(`"`xtitle'"' == `""', `"`varname'"', `"`xtitle'"')
graph twoway connected `yvars' `at' ///
, title(`title') ytitle(`ytitle') xtitle(`xtitle') `options'
end
exit

In our graph command, we have not specified any other options, such as colors or marker symbols. 
Therefore, apart from the explicitly defined defaults, the graph resembles a standard Stata graph. 
At the same time, we want to be able to define all of graph's characteristics whenever we call up 
the program. To do so, we have also specified the wildcard element in the syntax command. 
The star (*) means that the user can enter further options in addition to those that have been 
explicitly allowed. These options are saved by syntax in the local macro options and can then
be included in the graph command.

You should now try out these changes. Please type: */

drop xvals* f*
discard
denscomp linc, by(emp) at(xhelp) ti(My Graph) xtitle(My X-Title) ytitle (My Y-Title)

* ## Extended macro functions

/* Now all that is missing is an informative legend. This is a little tricky. 
By default, the variable labels of the data being plotted are used as the text in the legend. 
We can use this default setting by labeling the individual variables generated by kdensity 
with the appropriate caption. 
Ideally, the legend should inform us of the category of the variable in by(). 
We will therefore use the value label of the variable in by() as the text for the legend.

We can achieve this by using the extended macro function label. 
The general syntax for extended macro functions is 

local macroname: extended macro function

That is, using a colon instead of an equal-sign informs Stata that 
an extended macro function is to follow.

Here we add lines to the program, which concerns the foreach loop calculating 
the various density estimations. 
Also we slightly changed the command for displaying the graph.

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric) ///
[TItle(string asis) YTItle(string asis) XTItle(string asis) * ]
marksample touse
quietly levelsof `by' if `touse', local(K)
local i 1
foreach k of local K {
	kdensity `varlist' if `by'==`k' & `touse', gen(xvals`k' fx`k') at(`at') nograph
	local yvars "`yvars' fx`k'"
	local label: label (`by') `k'
	local leglab `" `leglab' label(`i++' `"`label'"') "'
}
local title = cond(`"`title'"' == `""', `"`varlist' by `by'"', `"`title'"')
local ytitle = cond(`"`ytitle'"' == `""', `"Density"', `"`ytitle'"')
local xtitle = cond(`"`xtitle'"' == `""', `"`varname'"', `"`xtitle'"')
graph twoway connected `yvars' `at' ///
, title(`title') ytitle(`ytitle') xtitle(`xtitle') legend(`leglab') `options'
end
exit

To customize the legend on a graph, we use the legend() option. 
If our graph had two data series, one for men and one for women, we could then specify
legend(label(1 "Men") label(2 "Women")). 
Here we do not know the number of categories in the by variable, 
so we use the local macro i to keep a count as we loop through them.

Before the loop begins, the macros label and leglab are empty. 
During the first iteration of the loop, the macro label is filled with the value label 
corresponding to the first level of the by() variable. 
The leglab macro is then set to label(1 "text"), where text is the text contained in label. 

The macro i is also incremented so that next time it will be equal to 2. 
In later iterations of the loop, the leglab macro is modified by adding elements 
of the form label(x "text"), where x is the value of the by() variable and text is the 
corresponding value label.

The local macro leglab can then easily be used for the definition of the legend in
the graph command.

With this change, the first raw version of the ado-file would now be ready. 
To check the situation so far, you should try the program again: */

discard
drop xvals* f*
denscomp linc, by(emp) at(xhelp)
drop xvals* f*
denscomp linc, by(edu) at(xhelp)

* ## Avoiding changes in the dataset

/* A program that generates variables is usually inconvenient. 
No one really enjoys deleting the variables generated by the previous command 
every time they use it. Therefore, an obvious solution would be to delete the newly generated 
variables at the end of every programs but this is not much better either, 
as it assumes that the program will actually run until it finishes. 
If the user aborts the program or if it crashes and displays an error message, 
the variables generated up until then will remain in the dataset.

The solution is to use temporary variables. You have already come across a temporal
variable when you were using marksample. 
Temporary variables are automatically removed when a program exits and are defined in two steps:

1. Declaring the variables' names
2. Generating the temporary variables

You declare temporary variables using the tempvar command, and you generate them using generate. 
However, the variable names are enclosed in single quotation marks, as is the case with local macros.

If, for example, you wanted to include the temporary variables xl and x2 in a program, you could code:

tempvar xl x2
gen `xl' = 1
gen `x2' = `xl'/_n

Here we generated variables within the kdensity command to hold the density estimations. 
To generate these variables as temporary variables, you declare them within the foreach loop 
and call these temporary variables in option gen() of the kdensity command:

program denscomp
version 10
syntax varname(numeric) [if] [in], by(varname) at(varname numeric) ///
[TItle(string asis) YTItle(string asis) XTItle(string asis) * ]
marksample touse
quietly levelsof `by' if `touse', local(K)
foreach k of local K {
	tempvar g`k'
	quitely gen `g`k'' = `by'==`k' if `by' < . & `touse'
	local G "`G' `g`k''"
}
local i 1
foreach k of local K {
	tempvar fx`k' xvals`k'
	kdensity `varlist' if `by'==`k' & `touse', gen(`xvals`k'' `fx`k'') at(`at') nograph
	local yvars "`yvars' fx`k'"
	local label: label (`by') `k'
	local leglab `" `leglab' label(`i++' `"`label'"') "'
}
local title = cond(`"`title'"' == `""', `"`varlist' by `by'"', `"`title'"')
local ytitle = cond(`"`ytitle'"' == `""', `"Density"', `"`ytitle'"')
local xtitle = cond(`"`xtitle'"' == `""', `"`varname'"', `"`xtitle'"')
graph twoway connected `yvars' `at' ///
, title(`title') ytitle(`ytitle') xtitle(`xtitle') legend(`leglab') `options'
end
exit

Note the two single quotation marks. The first indicates the end of the local macro k, 
whereas the second indicates the end of the temporary variable. 
You also need to replace every other instance of fx`k' in the program with `fx`k''. 
After this step, a first beta version of the ado-file is ready.

Beta version of denscomp.ado:

program denscomp
version 10.0
syntax varname(numeric) [if] [in], by(varname) at(varname numeric) ///
[TItle(string asis) YTItle(string asis) XTItle(string asis) * ]
marksample touse

// Store levels of By-Variable
quietly levelsof `by' if `touse', local(K)

// Check Graph-Options
if "`options'" ~= "" {
	foreach k of local K {
		tempvar g`k'
		quitely gen `g`k'' = `by'==`k' if `by' < . & `touse'
		local G "`G' `g`k''"
	}
	graph twoway scatter `G' `at', `options' nodraw
}

// Calculate Kernel-Densities
local i 1
foreach k of local K {
	tempvar fx`k' xvals`k'
	kdensity `varlist' if `by'==`k' & `touse', gen(`xvals`k'' `fx`k'') at(`at') nograph
	local yvars "`yvars' fx`k'"
	local label: label (`by') `k'
	local leglab `" `leglab' label(`i++' `"`label'"') "'
}

// Graph-Defaults
local title = cond(`"`title'"' == `""', `"`varlist' by `by'"', `"`title'"')
local ytitle = cond(`"`ytitle'"' == `""', `"Density"', `"`ytitle'"')
local xtitle = cond(`"`xtitle'"' == `""', `"`varname'"', `"`xtitle'"')

// Display the Graph
graph twoway connected `yvars' `at' ///
, title(`title') ytitle(`ytitle') xtitle(`xtitle') legend(`leglab') `options'
end
exit
****************************************************************************************************************************************************************

* ## Help files

Once you are satisfied with your ado, you should write a help file. 
Help files are straight forward ASCII files that are displayed on screen by Stata when one types

help command

For Stata to find the help file for a command, the help file must have the same name
as the ado-file, albeit with the file extension .sthlp. Here the help file would therefore
be called denscomp.sthlp.

For your help file to have the same look and feel as common Stata help files, 
you can use the State Markup and Control Language (SMCL). 
In Stata, all output is in SMCL format, be it normal command output or output from invoking a help file. 
In SMCL directives are used to affect how output appears. 
If, for example, your help text includes

{p 4 4 2}
{cmd:denscomp} uses {help kdensity} to estimate densities of varname
for categories of by().

when you call up help with help denscomp, the line "denscomp uses kdensity to estimate
densities of varname for categories of by()" will be displayed, 
with "denscomp" displayed in white and "kdensity" displayed in blue and clickable. 
That is, by clicking on kdensity, you will call up the help text for kdensity.

In [P] smcl, you will find further information on help, as well as recommendations
for organizing your help.
****************************************************************************************************************************************************************
